'use strict';

exports.__esModule = true;
exports.StreamDefaultProps = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; /*
                                                                                                                                                                                                                                                                   * This file is part of the nivo project.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * Copyright 2016-present, Raphaël Benitte.
                                                                                                                                                                                                                                                                   *
                                                                                                                                                                                                                                                                   * For the full copyright and license information, please view the LICENSE
                                                                                                                                                                                                                                                                   * file that was distributed with this source code.
                                                                                                                                                                                                                                                                   */


var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _lodash = require('lodash');

var _d3Shape = require('d3-shape');

var _d3Scale = require('d3-scale');

var _compose = require('recompose/compose');

var _compose2 = _interopRequireDefault(_compose);

var _pure = require('recompose/pure');

var _pure2 = _interopRequireDefault(_pure);

var _withPropsOnChange = require('recompose/withPropsOnChange');

var _withPropsOnChange2 = _interopRequireDefault(_withPropsOnChange);

var _defaultProps = require('recompose/defaultProps');

var _defaultProps2 = _interopRequireDefault(_defaultProps);

var _Nivo = require('../../../Nivo');

var _Nivo2 = _interopRequireDefault(_Nivo);

var _props = require('../../../props');

var _colorUtils = require('../../../lib/colorUtils');

var _SvgWrapper = require('../SvgWrapper');

var _SvgWrapper2 = _interopRequireDefault(_SvgWrapper);

var _Container = require('../Container');

var _Container2 = _interopRequireDefault(_Container);

var _Axes = require('../../axes/Axes');

var _Axes2 = _interopRequireDefault(_Axes);

var _Grid = require('../../axes/Grid');

var _Grid2 = _interopRequireDefault(_Grid);

var _StreamLayers = require('./StreamLayers');

var _StreamLayers2 = _interopRequireDefault(_StreamLayers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var stackMin = function stackMin(layers) {
    return (0, _lodash.min)(layers.reduce(function (acc, layer) {
        return [].concat(acc, layer.map(function (d) {
            return d[0];
        }));
    }, []));
};
var stackMax = function stackMax(layers) {
    return (0, _lodash.max)(layers.reduce(function (acc, layer) {
        return [].concat(acc, layer.map(function (d) {
            return d[1];
        }));
    }, []));
};

var Stream = function Stream(_ref) {
    var data = _ref.data,
        keys = _ref.keys,
        order = _ref.order,
        offsetType = _ref.offsetType,
        curve = _ref.curve,
        margin = _ref.margin,
        width = _ref.width,
        height = _ref.height,
        fullWidth = _ref.fullWidth,
        fullHeight = _ref.fullHeight,
        axisTop = _ref.axisTop,
        axisRight = _ref.axisRight,
        axisBottom = _ref.axisBottom,
        axisLeft = _ref.axisLeft,
        enableGridX = _ref.enableGridX,
        enableGridY = _ref.enableGridY,
        theme = _ref.theme,
        color = _ref.color,
        fillOpacity = _ref.fillOpacity,
        animate = _ref.animate,
        motionStiffness = _ref.motionStiffness,
        motionDamping = _ref.motionDamping,
        isInteractive = _ref.isInteractive;

    var stack = (0, _d3Shape.stack)().keys(keys).offset((0, _props.stackOffsetFromProp)(offsetType)).order((0, _props.stackOrderFromProp)(order));

    var layers = stack(data);

    var minValue = stackMin(layers);
    var maxValue = stackMax(layers);

    var xScale = (0, _d3Scale.scalePoint)().domain((0, _lodash.range)(data.length)).range([0, width]);
    var yScale = (0, _d3Scale.scaleLinear)().domain([minValue, maxValue]).range([height, 0]);

    var area = (0, _d3Shape.area)().x(function (d, i) {
        return xScale(i);
    }).y0(function (d) {
        return yScale(d[0]);
    }).y1(function (d) {
        return yScale(d[1]);
    }).curve((0, _props.curveFromProp)(curve));

    var enhancedLayers = layers.map(function (layer, i) {
        return {
            id: keys[i],
            layer: layer,
            path: area(layer),
            color: color(i)
        };
    });

    var motionProps = {
        animate: animate,
        motionDamping: motionDamping,
        motionStiffness: motionStiffness
    };

    return _react2.default.createElement(
        _Container2.default,
        { isInteractive: isInteractive },
        function (_ref2) {
            var showTooltip = _ref2.showTooltip,
                hideTooltip = _ref2.hideTooltip;
            return _react2.default.createElement(
                _SvgWrapper2.default,
                { width: fullWidth, height: fullHeight, margin: margin },
                _react2.default.createElement(_Grid2.default, _extends({
                    theme: theme,
                    width: width,
                    height: height,
                    xScale: enableGridX ? xScale : null,
                    yScale: enableGridY ? yScale : null
                }, motionProps)),
                _react2.default.createElement(_StreamLayers2.default, _extends({
                    layers: enhancedLayers,
                    area: area,
                    fillOpacity: fillOpacity,
                    showTooltip: showTooltip,
                    hideTooltip: hideTooltip
                }, motionProps)),
                _react2.default.createElement(_Axes2.default, _extends({
                    xScale: xScale,
                    yScale: yScale,
                    width: width,
                    height: height,
                    theme: theme,
                    top: axisTop,
                    right: axisRight,
                    bottom: axisBottom,
                    left: axisLeft
                }, motionProps))
            );
        }
    );
};

Stream.propTypes = _extends({
    // data
    data: _propTypes2.default.arrayOf(_propTypes2.default.object).isRequired,
    keys: _propTypes2.default.array.isRequired,

    order: _props.stackOrderPropType.isRequired,
    offsetType: _props.stackOffsetPropType.isRequired,
    curve: _props.areaCurvePropType.isRequired,

    // dimensions
    width: _propTypes2.default.number.isRequired,
    height: _propTypes2.default.number.isRequired,
    margin: _props.marginPropType,

    // axes & grid
    axisTop: _propTypes2.default.object,
    axisRight: _propTypes2.default.object,
    axisBottom: _propTypes2.default.object,
    axisLeft: _propTypes2.default.object,
    enableGridX: _propTypes2.default.bool.isRequired,
    enableGridY: _propTypes2.default.bool.isRequired,

    // theming
    theme: _propTypes2.default.object.isRequired,
    colors: _propTypes2.default.any.isRequired,
    fillOpacity: _propTypes2.default.number.isRequired,
    color: _propTypes2.default.func.isRequired

}, _props.motionPropTypes, {

    isInteractive: _propTypes2.default.bool
});

var StreamDefaultProps = exports.StreamDefaultProps = {
    order: 'none',
    offsetType: 'wiggle',
    curve: 'catmullRom',

    // dimensions
    margin: _Nivo2.default.defaults.margin,

    // axes & grid
    axisBottom: {},
    enableGridX: true,
    enableGridY: false,

    // theming
    theme: {},
    colors: 'nivo',
    fillOpacity: 1,

    // motion
    animate: true,
    motionStiffness: _Nivo2.default.defaults.motionStiffness,
    motionDamping: _Nivo2.default.defaults.motionDamping
};

var enhance = (0, _compose2.default)((0, _defaultProps2.default)(StreamDefaultProps), (0, _withPropsOnChange2.default)(['theme'], function (_ref3) {
    var theme = _ref3.theme;
    return { theme: (0, _lodash.merge)({}, _Nivo.defaultTheme, theme) };
}), (0, _withPropsOnChange2.default)(['colors'], function (_ref4) {
    var colors = _ref4.colors,
        colorBy = _ref4.colorBy;
    return {
        color: (0, _colorUtils.getColorRange)(colors)
    };
}), (0, _withPropsOnChange2.default)(function (props, nextProps) {
    return props.data !== nextProps.data || props.stacked !== nextProps.stacked || props.width !== nextProps.width || props.height !== nextProps.height || !(0, _lodash.isEqual)(props.margin, nextProps.margin);
}, function (_ref5) {
    var data = _ref5.data,
        fullWidth = _ref5.width,
        fullHeight = _ref5.height,
        _margin = _ref5.margin;

    var margin = Object.assign({}, _Nivo2.default.defaults.margin, _margin);
    var width = fullWidth - margin.left - margin.right;
    var height = fullHeight - margin.top - margin.bottom;

    return {
        margin: margin,
        width: width,
        height: height,
        fullWidth: fullWidth,
        fullHeight: fullHeight
    };
}), _pure2.default);

exports.default = enhance(Stream);